function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

/*
 * Created Date: 2019-07-09
 * Author: 宋慧武
 * ------
 * Last Modified: Tuesday 2019-07-09 18:09:50 pm
 * Modified By: the developer formerly known as 宋慧武 at <songhuiwu001@ke.com>
 * ------
 * HISTORY:
 * ------
 * Javascript will save your soul!
 */
/**
 * @desc 获取 DOM CSS 属性的值
 *
 * @param {DOMElement} ele A DOM 元素
 * @returns {String}
 */

function getStylePropValue(ele, prop) {
  return window.getComputedStyle(ele).getPropertyValue(prop);
}
/**
 * @desc 元素是否隐藏
 *
 * @param {DOMElement} ele A DOM 元素
 * @returns {Boolean} true => 未隐藏可见  false => 隐藏不可见
 */

function isVisible(ele) {
  if (ele === window.document) {
    return true;
  }

  if (!ele || !ele.parentNode) {
    return false;
  }

  var parent = ele.parentNode;
  var visibility = getStylePropValue(ele, "visibility");
  var display = getStylePropValue(ele, "display");

  if (visibility === "hidden" || display === "none") {
    return false;
  }

  return parent ? isVisible(parent) : true;
}

/*
 * Created Date: 2019-07-11
 * Author: 宋慧武
 * ------
 * Last Modified: Saturday 2019-07-13 12:24:04 pm
 * Modified By: the developer formerly known as 宋慧武 at <songhuiwu001@ke.com>
 * ------
 * HISTORY:
 * ------
 * Javascript will save your soul!
 */

/**
 * @desc 获取对象的键值
 *
 * @param {Object} value
 * @returns {Array} [keys, values]
 */
function zipObject(value) {
  return [Object.values(value), Object.keys(value)];
}
/**
 * @desc 判断给定变量是否为一个函数
 *
 * @param {*} v
 * @return {Boolean}
 */

var isFun = function isFun(v) {
  return typeof v === "function" || false;
};
/**
 * @desc chenck 埋点ID是否合法
 *
 * @param {Object} value
 * @returns {Array} [keys, values]
 */

var vaildEvent = function vaildEvent(events, eventId) {
  if (!isFun(events[eventId])) {
    throw new Error("track eventId '".concat(eventId, "' does not exist"));
  }
};
/**
 * @desc check 异步埋点监听key是否存在
 *
 * @param {Object} value
 * @returns {Array} [keys, values]
 */

var vaildWatchKey = function vaildWatchKey(stateKey, propKey) {
  if (!stateKey && !propKey) {
    throw new Error("Missing arguments.{stateKey} or arguments.{propKey} in async track");
  }
};

var ONCE = "once";
var modifiers = {
  CLICK: "click",
  CLICK_AFTER: "click.after",
  ASYNC: "async",
  ASYNC_DELAY: "async.delay"
};
/**
 * @desc inject 装饰器
 * @param {Function} reaction 如果为 Mobx Class 则需要注入 mobx.reaction 来监听通过异步action 对 state 的改变
 * @param {Object} trackEvents 当前页面需要的埋点事件
 */

var inject = function inject() {
  for (var _len = arguments.length, props = new Array(_len), _key = 0; _key < _len; _key++) {
    props[_key] = arguments[_key];
  }

  return function (target) {
    Object.assign.apply(Object, [target.prototype].concat(props));
  };
};
/**
 * @desc track 埋点装饰器
 * @param {String} modifier 修饰符，对应的埋点类型
 * @param {Number | String} eventId 埋点事件id
 * @param {Object} params 自定义参数，目前支持tateKey、propKey、delay、ref
 *
 * @property[stateKey] 对应组件 state 或者 mobx observable 中 key
 * @property[propKey] 对应组件 props 中 key
 * @property[delay] 埋点延迟时间
 * @property[ref] 对应 DOM 的引用，避免小于埋点延迟时间内发生DOM影藏造成埋点继续上报的问题
 */

function track(modifier, eventId) {
  var params = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var _zipObject = zipObject(modifiers),
      _zipObject2 = _slicedToArray(_zipObject, 1),
      mdfs = _zipObject2[0];

  if (!mdfs.includes(modifier.replace(/\.once/g, ""))) {
    throw new Error("modifier '".concat(modifier, "' does not exist"));
  }

  return function (_, name, descriptor) {
    var handler;
    var value = descriptor.value,
        initializer = descriptor.initializer; // 事件行为埋点

    if (modifier.includes(modifiers.CLICK)) {
      handler = function handler() {
        var _this = this;

        for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          args[_key2] = arguments[_key2];
        }

        var context = this;
        var isRC = this.isReactComponent; // 是否为 react 组件

        var once = modifier.includes(ONCE);
        var onceProp = "".concat(name, "_").concat(eventId);
        var after = modifier.includes(modifiers.CLICK_AFTER);

        var tck = function tck() {
          var _this$trackEvents$eve;

          if (_this[onceProp]) return; // 如果存在once修饰符，且为true则直接返回

          vaildEvent(_this.trackEvents, eventId); // 检测eventId是否合法

          isRC && (context = _objectSpread({}, _this.state, _this.props));

          (_this$trackEvents$eve = _this.trackEvents[eventId]).call.apply(_this$trackEvents$eve, [null, context].concat(args));

          once && (_this[onceProp] = true);
        };

        var fn = value ? value.bind(this) : initializer.apply(this, args);
        var queue = after ? [fn, isRC ? function () {
          return _this.setState({}, tck);
        } : tck] : [tck, fn];
        return queue.forEach(function (sub) {
          return sub.apply(void 0, args);
        });
      };
    } // 异步行为埋点
    else {
        handler = function handler() {
          var _this2 = this;

          for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
            args[_key3] = arguments[_key3];
          }

          var tck;
          var context = this;
          var isRC = this.isReactComponent; // 是否为 react 组件

          var once = modifier.includes(ONCE);
          var fn = value ? value.bind(this) : initializer.apply(this, args);
          var stateKey = params.stateKey,
              propKey = params.propKey;
          var watchPropKey = "".concat(name, "_").concat(propKey, "_").concat(eventId); // 保证key的唯一性

          var watchStateKey = "".concat(name, "_").concat(stateKey, "_").concat(eventId);
          vaildEvent(this.trackEvents, eventId);
          vaildWatchKey(stateKey, propKey);
          !this.tckQueue && (this.tckQueue = {}); // 在当前实例维护一个异步埋点队列

          !this.tckQueuePropKeys && (this.tckQueuePropKeys = []); //

          !this.tckQueueStateKeys && (this.tckQueueStateKeys = []); //

          if (modifier === modifiers.ASYNC_DELAY || modifier === modifiers.ASYNC_DELAY + ".once") {
            tck = function tck() {
              var _params$delay = params.delay,
                  delay = _params$delay === void 0 ? 0 : _params$delay,
                  ref = params.ref;
              var ele = _this2[ref] || document;
              _this2.$timer = setTimeout(function () {
                var _this2$trackEvents$ev;

                isRC && (context = _objectSpread({}, _this2.state, _this2.props));
                isVisible(ele) && (_this2$trackEvents$ev = _this2.trackEvents[eventId]).call.apply(_this2$trackEvents$ev, [null, context].concat(args));
                clearTimeout(_this2.$timer);
              }, delay);
            };
          } else if (modifier === modifiers.ASYNC || modifier === modifiers.ASYNC + ".once") {
            tck = function tck() {
              var _this2$trackEvents$ev2;

              isRC && (context = _objectSpread({}, _this2.state, _this2.props));
              if (propKey && _this2["".concat(watchPropKey, "_").concat(ONCE)]) return;
              if (stateKey && _this2["".concat(watchStateKey, "_").concat(ONCE)]) return;

              (_this2$trackEvents$ev2 = _this2.trackEvents[eventId]).call.apply(_this2$trackEvents$ev2, [null, context].concat(args));

              once && propKey && (_this2["".concat(watchPropKey, "_").concat(ONCE)] = true);
              once && stateKey && (_this2["".concat(watchStateKey, "_").concat(ONCE)] = true);
            };
          }

          propKey && (this.tckQueue[watchPropKey] = [tck]) && this.tckQueuePropKeys.push(watchPropKey);
          stateKey && (this.tckQueue[watchStateKey] = [tck]) && this.tckQueueStateKeys.push(watchStateKey);

          if (isRC && !this.getSnapshotBeforeUpdate) {
            this.getSnapshotBeforeUpdate = function (prevProps, prevState) {
              Object.keys(_this2.tckQueue).forEach(function (watchKey) {
                var oldVal, newVal, cbks, key;

                if (_this2.tckQueuePropKeys.includes(watchKey)) {
                  key = watchKey.split("_")[1];
                  newVal = _this2.props[key];
                  oldVal = prevProps[key];
                  cbks = _this2.tckQueue[watchKey];
                }

                if (_this2.tckQueueStateKeys.includes(watchKey)) {
                  key = watchKey.split("_")[1];
                  newVal = _this2.state[key];
                  oldVal = prevState[key];
                  cbks = _this2.tckQueue[watchKey];
                }

                oldVal !== newVal && cbks.forEach(function (sub) {
                  return sub && sub();
                });
              });
              return null;
            };
          } else {
            if (this.reaction) {
              var cbks = this.tckQueue[watchStateKey];
              var disposer = this.reaction(function () {
                return _this2[stateKey];
              }, function () {
                cbks.forEach(function (sub) {
                  return sub && sub();
                });
                disposer();
              });
            }
          }

          return fn.apply(void 0, args);
        };
      }

    if (value) {
      descriptor.value = function () {
        for (var _len4 = arguments.length, args = new Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
          args[_key4] = arguments[_key4];
        }

        return handler.apply(this, args);
      };
    } // 兼容箭头函数 https://github.com/MuYunyun/diana/issues/7


    if (initializer) {
      descriptor.initializer = function () {
        return handler;
      };
    }
  };
}

export { inject, track };
